/****************************************************/
/*                                                  */
/* TGSTM32F4.h : STM32F4-Discovery tone generator   */
/*                                                  */
/* NON COMMERCIAL USE ONLY                          */
/*                                                  */
/* 2012/04/15 for STM32F4-Discovery                 */
/* 2011/04/29 Rev. 1.2 suport stereo, SMF sequencer */
/* 2011/04/24 Rev. 1.1 by pcm1723                   */
/*                                                  */
/****************************************************/
#ifndef TGSTM32F4_H
#define TGSTM32F4_H

#if defined(STM32F4XX) // STM32F4-Discovery
//
// compile time global options
//
// define "MIDIONLY" for disable DEMO playback
//
// #define MIDIONLY
// #define DEMOONLY

// define   "USE_SMF" for SMF  sequencer (V1.2 new feature)
// undefine "USE_SMF" for SMAF sequencer (V1.1 or older compatible)
#define USE_SMF 1

#define MAXSEQCH      32
#define ABS_MAX_UNITS 32
#define ABS_MAX_PARTS 32

//
// STEREO_MODE = 1 for stereo
// STEREO_MODE = 0 for monaural
#define STEREO_MODE 1

#define USEWAVEQUE
#define WAVEQUESIZE 512

#define debug 0

#define REVERB   1
#if (0)
#define Shroeder 1
#define CHORUS   1
#endif

  #define DELAY_LEN1_MAX 1913
  #define DELAY_LEN2_MAX 1733
  #define DELAY_LEN3_MAX 1597
  #define DELAY_LEN4_MAX 1447
  #define DELAY_LEN5_MAX  241
  #define DELAY_LEN6_MAX   83

  #define DELAY_COEF_SHIFT 12


// #define FORCE_LEGACY_MIDI
// #define FORCE_SERIAL_MIDI

//#define FORCE_LOW

//
// LCD related
//
#define LCD
#define LCD_4bit 1
#define BANNER_STRING "STM32F407   MIDIFM ToneGen V1.2 "

//
// demo / sequencer related
//
#ifndef MIDIONLY
  #define SEQUENCER
#endif

#define EXTRADEMO 1

#define DEFAULT_SYS_INDEX 1

#define ALIGN4 __attribute__ ((aligned(4)))

//
// digital delay definition
//
#define DEFAULT_DELAY_ms 120 /* 120 */
#define DELAYMEMSIZE (50*DEFAULT_DELAY_ms)

#define MAX_CONFIG 5

#define AUDIO_SAMPLES_PER_DMA_BUF (8) /* 8 */
// factor '2' for 'stm32f4_discovery_audio_codec.c/.h' V1.0.0 (19-Sep-2011)
#define DMA_BUF_SIZE_ADJ (2)
// factor '4' for 'stm32f4_discovery_audio_codec.c/.h' V1.1.0 (28-Oct-2011)
//#define DMA_BUF_SIZE_ADJ (4)

// DMA buffer for audio samples
// factor 2 for STEREO sample
// factor 2 for 16-bit (2-byte) data
#define DMA_BUF_BYTES (2*2*AUDIO_SAMPLES_PER_DMA_BUF)
typedef uint16_t dac_dma_buf_t[2*AUDIO_SAMPLES_PER_DMA_BUF];
extern dac_dma_buf_t dac_dma_buf[2]; // double buffer
//
// machine dependent defines
//
#define LED_ON      STM_EVAL_LEDOn(LED4)
#define LED_OFF     STM_EVAL_LEDOff(LED4)
#define RCV_LED_ON  STM_EVAL_LEDOn(LED6)
#define RCV_LED_OFF STM_EVAL_LEDOff(LED6)
#define USER_KEY    STM_EVAL_PBGetState(BUTTON_USER)
//
// LCD / SW define
//
#define LCD_RSBIT         (0x10)
#define SW_RE_BITMASK     0x03
#define SW_RE_DETENT      0x03
#define SW_RE_FWD         0x02
#define SW_RE_BACK        0x01
//
// use USART3_TX on PD8 for MIDI Tx
//
#define MIDI_TX_UART USART3
#define MIDI_TX_PORT GPIOD
#define MIDI_TX_BIT  8
#define MIDI_TX_AF   GPIO_AF_USART3
//
// use USART3_RX on PD9 for MIDI Rx
//
#define MIDI_RX_UART USART3
#define MIDI_RX_PORT GPIOD
#define MIDI_RX_BIT  9
#define MIDI_RX_AF   GPIO_AF_USART3
//
// use PD11 for (legacy) MIDI/seial MIDI sense
//
#define MIDI_SENS_PORT GPIOD
#define MIDI_SENS_BIT  11
//
// use USART2_TX on PA2 for MCU LCD I/F Tx
//
#define LCD_TX_UART USART2
#define LCD_TX_PORT GPIOA
#define LCD_TX_BIT  2
#define LCD_TX_AF   GPIO_AF_USART2
//
// use USART2_RX on PD6 for MCU LCD I/F Rx
//
#define LCD_RX_UART USART2
#define LCD_RX_PORT GPIOD
#define LCD_RX_BIT  6
#define LCD_RX_AF   GPIO_AF_USART2
//
// use ADC1 unit for CPU loading measure
//
#define ADC_UNIT ADC1
#define ADC_CH   ADC_Channel_15
#define ADC_RCC  RCC_APB2Periph_ADC1
//
// use ADC12_IN15 analog input on PC5
//
#define ADC_IN_PORT GPIOC
#define ADC_IN_BIT  5
// ADC functions
#define READ_ADC  (0x0FFF ^ ADC_GetConversionValue(ADC_UNIT))
#define START_ADC (ADC_SoftwareStartConv(ADC_UNIT))
// Key SW USART functions
#define KEYSW_OVRUN 0
#define CLR_KEYSW_OVRUN
#define CHECK_KEYSW (USART_SR_RXNE & (LCD_RX_UART->SR))
#define GET_KEYSW   (LCD_RX_UART->DR)
// send data to LCD
#define PUT_LCD(x)  (LCD_TX_UART->DR = (0xe0 | (x)))
#if (defined(DEMOONLY))
  #define CHECKMIDI (0x00)
#else
  #define CHECKMIDI (USART_SR_RXNE & (MIDI_RX_UART->SR))
#endif // #ifdef DEMOONLY
#define GETMIDI    (MIDI_RX_UART->DR)
#define PUTMIDI(x) (MIDI_TX_UART->DR = x)
#define MIDI_OVRUN (0)
#define CLR_MIDI_OVRUN
#define ROMTABLE __attribute__ ((__section__(".rodata.top"), __aligned__(0x200)))
#define GET_PANEL_KEY (0x01 ^ USER_KEY)

extern void starm_port_setup( void );
extern void stm32_port_setup( void );

#include "../common/globals.h"

#define WAVE_QUE_HAS_ROOM (wq_wr_ix != wq_rd_ix)
#define UPDATE_QUE_INDEX(ix, n) (ix = ((ix + (n)) & (WAVEQUESIZE - 1)))
#define UPDATE_QUE_READ_IX(n)  UPDATE_QUE_INDEX(wq_rd_ix, n)
#define UPDATE_QUE_WRITE_IX(n) UPDATE_QUE_INDEX(wq_wr_ix, n)

static __inline__ void ENQUEUE(uint16_t Lch, uint16_t Rch)
{
  wave_que[wq_wr_ix][0] = Lch;
#if (STEREO_MODE)
  wave_que[wq_wr_ix][1] = Rch;
#else
  wave_que[wq_wr_ix][1] = Lch;
#endif
  UPDATE_QUE_WRITE_IX(1);
} // void ENQUEUE()

static __inline__ void INIT_QUE_IX( void )
{
  wq_wr_ix = 0;
  wq_rd_ix = 0;
} // void INIT_QUE_IX()

#endif // #if defined(STM32F4XX)
#endif // #ifndef TGSTM32F4_H
