/*****************************************/
/* globals.c : global variable instances */
/*****************************************/

#include "../common/tonegen.h"

#if (defined(__RX) | defined(STM32F4XX))
volatile int fastchip = (1 - STEREO_MODE);
#endif


#if defined(LCD)
char banner_str[] = BANNER_STRING;

const uint8_t cgram[CGRAMSIZE] = {
//  0x01, 0x01, 0x01, 0x01, 0x01, 0x0f, 0x1f, 0x0e, // quarter
//  0x00, 0x00, 0x00, 0x00, 0x1f, 0x0e, 0x0e, 0x00, // rest
  0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x1f, // C
  0x10, 0x10, 0x10, 0x17, 0x17, 0x17, 0x10, 0x1f, // C, Key ON
  0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x04, 0x04, 0x1f, // C#, D#, F#, G#, A#
  0x1f, 0x1b, 0x11, 0x1b, 0x1f, 0x04, 0x04, 0x1f, // C#, D#, F#, G#, A#, Key ON
  0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x1f, // E, B
  0x01, 0x01, 0x01, 0x1d, 0x1d, 0x1d, 0x01, 0x1f, // E, B, Key ON
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, // D, F, G, A
  0x00, 0x00, 0x00, 0x0e, 0x0e, 0x0e, 0x00, 0x1f  // D, F, G, A, Key ON
};
#endif 

#if defined(SEQUENCER)

/* Demo songs */
  #if defined(USE_SMF)
  
const demo_st demosong[] = {
  {arabe_sp, 
   {46, 46, 46, 46}, // inst = Orchestral Harp
   "C.A.Debussy :\n\"Arabesque 1\"   "},
}; // const demo_st demosong[]

  #else // #if defined(USE_SMF)
  
const demo_st demosong[] = {
    #ifdef EXTRADEMO
  {Chopin66, 
   {1, 1, 1, 1}, // inst = Electric grand piano
   "Chopin:Fantaisie-Impromptu Op.66"},
    #endif
  {arabesq1, 
   {46, 46, 46, 46}, // inst = Orchestral Harp
   "C.A.Debussy :\n\"Arabesque 1\"   "},
    #ifdef EXTRADEMO
  {invent13,
   {5, 5, 5, 5},     // inst = E.Piano1
   "J.S.Bach:BWV.784\n\"Inventionen 13\""},
    #endif
}; // const unsigned char* demosong[]

  #endif // #if defined(USE_SMF)
#endif // #if defined(SEQUENCER)

// Operator mult code to actual multiplier conversion
const int mult_tab[16] = {
// code:       0   1  2  3  4   5   6   7   8   9  10  11  12  13  14  15
// multiplier: 0.5 1  2  3  4   5   6   7   8   9  10  10  12  12  15  15
               1,  2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 20, 24, 24, 30, 30
}; // const int mult_tab[]

// LFO increment table for 4ms tick
// value = 0x10000 * tick_period * LFO_freq
const int LFO_inc_tab[4] = {
// LFO freq:  1.9Hz 4.2Hz 6.1Hz 7.2Hz
              498,  1101, 1599, 1887
              
}; // const int LFO_inc_tab[4]

int MAXMIDICH;
int MAXUNITS;
int SEQTICK1ms;
int SEQTICK4ms;
int FS_ASC;
uint seq_duration ;
int seq_tick;
int tick1ms  = 0;
int t4ms     = 0;
int8_t midithru = 0;

#if defined(SEQUENCER)
  int demoflag = 1;
  int loopflag = 0;
  int demo_no  = 0;
  int demosongs = sizeof(demosong) / sizeof (demo_st);
#endif

int basech = 0;
int eos;
int panel_mode;
int last_note;
int midi_mon_part;
int sys_mode_index;
int oct_ofs;
int note_ofs;
int debounce;
uint32_t inc_sw_q, dec_sw_q;
uint16_t inc_sw_rl, dec_sw_rl;
int8_t  sw_val;
int8_t  re_state;
uint32_t panel_sw_q;
int8_t sw_val;
int demosave;
int8_t  serial_midi = 0;

volatile int t1_tick = 0;

#if defined(USEWAVEQUE)
volatile int wq_wr_ix, wq_rd_ix;
  #if defined(STM32F4XX) // always 2 channel
    int16_t wave_que[WAVEQUESIZE][2];
  #else
    int16_t wave_que[WAVEQUESIZE][STEREO_MODE+1];
  #endif // #if defined(STM32F4XX)
#endif // #if defined(USEWAVEQUE)

#if defined(syn_CV)
volatile int spiq_wr_ix, spiq_rd_ix;
int8_t spi_que[SPIQUESIZE];
#endif

// convert 4-bit keycode to 6dB/oct key scaling value
// KSL6.0dB = -(n - 32)/2 [dB]
// KSL6.0dB(in lb_Q6 format) = (64/6)*(n - 32)/2 = (64/12)*(n - 32)
//                           = (64/12)*(12*oct + k - 32)
//                           = 64*(oct - 3) + (16/3)*(k + 4)
const uint8_t ks60tab[12] = {
   (16*4)/3,   (16*5)/3,   (16*6)/3,   (16*7)/3,  // C ,C#,D ,D#
   (16*8)/3,   (16*9)/3,   (16*10)/3,  (16*11)/3, // E ,F ,F#,G
   (16*12)/3,  (16*13)/3,  (16*14)/3,  (16*15)/3  // G#,A ,A#,B
}; // unsigned char ks60tab[] 

unit_prop_t unit_prop[ABS_MAX_UNITS];

ch_prop_t ch_prop[MAXSEQCH];

#if defined(LCD)
uint8_t lcd_buf[LCD_BUFSIZE];
uint8_t lcd_buf2[LCD_BUFSIZE];
uint8_t *lcd_buf_p;
uint8_t *lcd_buf_w;
uint8_t lcd_row, lcd_col, lcd_buf_ind;
int16_t lcd_disp_timer;
uint8_t lcd_disp_row, lcd_disp_col, lcd_cur_pos;
int16_t lcd_blink_timer;
uint8_t lcd_blink_flag;
volatile uint16_t adc_data;
#endif

master_prop_t master_prop;
int sysx_cnt;

uint8_t sysx_buf[MAXSYSEXLEN];

#if defined(REVERB)
int     delay_mem_size;
int16_t delay_ms = DEFAULT_DELAY_ms;
//int16_t delaymem[DELAYMEMSIZE];
int echo_dep, delayp;
int16_t delaymem[DELAYMEMSIZE];
#endif // #if defined(REVERB)

#if defined(SEQUENCER)
void demo_off(void)
{
  if (demoflag | demosave) {
    int i;
    for (i = 0; i <MAXMIDICH; i++) 
    all_note_off(&(ch_prop[i]));
//   GM_system_on();  // all note off and init 
#if defined(LCD)
    lcd_buf_p = lcd_buf; // restore default screen
    lcd_draw_panel(panel_mode);
#endif
    demoflag = 0;
    demosave = 0;
  } // if (demoflag)
} // void demo_off()

//
// for SMF sequencer
//
#if defined(USE_SMF)
 int seq_mode;
 int32_t seq_time_inc;
 int32_t seq_targ_time;
uint32_t smf_tempo_us; // SMF tempo in microseconds 
uint32_t smf_tpqn; // SMF Ticks Per Quater Note
uint32_t mod_tpqn; // modified (factorized) TPQN
 int32_t mod_seq_tick_in_us; 
uint32_t seq_dt;
demo_st  *dp;
#endif // #if defined(USE_SMF)

#endif
