/**************************************************************/
/*                                                            */
/* stm32_hw.c  -- Hardware dependent routines                 */ 
/* for STMicroelectronics STM32F103xx (Cortex-M3 core)        */
/* and STMicroelectronics STM32F4-Discovery (Cortex-M4F core) */
/*                                                            */
/* NON COMMERCIAL USE ONLY                                    */
/*                                                            */
/* 2012/04/06 support STM32F4 Discovery                       */
/* 2010/09/19 modified for STBee / STBee Mini board           */
/* 2008/04/19 Rev. 1.01 by pcm1723                            */
/*                                                            */
/**************************************************************/

#include "../common/tonegen.h"
#include <string.h>

#if (defined(STM32F103) | defined(STM32F4XX))

#if defined(STM32F4XX)
#include "stm32f4_discovery.h"
#include "stm32f4_discovery_audio_codec.h"
dac_dma_buf_t dac_dma_buf[2]; // double buffer
#endif

//int  fastchip;

//
// speed_test() : determine speed of the chip
//                returns 1 for fast chip, 0 for slow chip
//
// measure execution cycle count of THUMB mode instructions
// using T1 counter
//
// __asm{} loop excution count is (15 * 1024) cycle for fast chip (I30 ?)
//                                (22 * 1024) cycle for slow chip (H3T ?)
#if (0)
int speed_test()
{
#if defined(FORCE_SLOW)
  return(0); // force slow chip configuration
#elif defined(FORCE_FAST)
  return(1); // force fast chip configuration
#else
  int k;
  T1CON = 0x0000; // halt T1 counter
  T1CON = 0x0180; // start T1 with HCLK, up-count, free-run
  k = T1VAL;      // remember inital T1 value
  __asm {
           MOV         R0,#0x0   ; for (i = 0; i < 1024; i++) \
                                 ; 15 cycle/pass for fast chip
      L_1: ADD         R0,#0x1   ; i++
           MOV         R1,R0     ; i
           LDR         R3,=0x400 
           CMP         R1,R3     ; i < 1024
           BLT         L_1    
  } // __asm
  k = T1VAL - k;           // calc elapsed clock count
  T1CON = 0x0000;          // halt T1 counter
  return (15 >= (k >> 10)); // compare with 15 cycle
#endif
} // int speed_test()
#endif
// Timer3 IRQ handler
void TIM3_IRQHandler(void)
{
  int32_t lw;
  if (TIM_FLAG_Update & (TIM3->SR)) {  // UIF set?
    t1_tick = 1;    // set timer1 tick flag
#ifdef USEWAVEQUE
#ifdef DEMOONLY	
    lw = (wave_que[wq_rd_ix] + (1 << 5)) >> 6; // rounding
    if     (-498 >= lw) lw = -498;
	else if (498 <= lw) lw =  498;
    TIM4->CCR3 = 500 + lw;  // output wave to Timer4 Ch3 
#else
    lw = wave_que[wq_rd_ix][0]; // get wave data
#if (0)
    SPI2->DR = lw; // output 16-bit data to Serial DAC
    lw = (unsigned long)(0x8000 ^ (wave_que[wq_rd_ix] + (1 << 5))) >> 6; // adjust for DAC bit allocation
    TIM4->CCR3 = 0x3FF & lw;  // output wave to Timer4 Ch3
#else
    DAC->DHR12L1 = (0x8000 ^ lw); // flip MSB for 12 bit DAC
#endif
#endif
    wq_rd_ix = (wq_rd_ix + 1) & (WAVEQUESIZE-1);
#endif
    TIM3->SR &= ~0x01;  // clear UIF flag
  } // if (TIM3->SR ...
} // void IRQ_Handler()

//
// I2S PLL setup for the PLL_N and PLL_R
//
void stm32_PLLI2S_MCLK_setup(uint32_t plln, uint32_t pllr)
{
// disable PLLI2S for parameter setting
	RCC_PLLI2SCmd(DISABLE);
// PLLI2S clock used as I2S clock source
	RCC_I2SCLKConfig(RCC_I2S2CLKSource_PLLI2S);
// Configure PLLI2S
	RCC_PLLI2SConfig(plln, pllr);
// Enable PLLI2S
	RCC_PLLI2SCmd(ENABLE);
// Wait till PLLI2S is ready
    while((RCC->CR & RCC_CR_PLLI2SRDY) == 0) {}
} // stm32_PLLI2S_MCLK_setup()

void clear_buffers( void )
{
#if defined(REVERB)
  bzero(delaymem, sizeof(delaymem));
#endif
  bzero(wave_que, sizeof(wave_que));
#if defined(STM32F4XX)
  bzero(dac_dma_buf, sizeof(dac_dma_buf));
#endif
}

void set_config(int mode)
{
#if defined(USE_12BIT_DAC)
  TIM3->ARR  = sys_config[fastchip][mode].t1div - 1; // set Timer 3 divisor constant
  TIM3->CCR4  = (sys_config[fastchip][mode].t1div) >> 1; // set Timer 3 duty
  #if (defined(STM32F4XX))
  TIM4->ARR  = TIM3->ARR; // set Timer 4 divisor constant
  #endif
#else // CS43L22 DAC
  #if defined(STM32F4XX)
  EVAL_AUDIO_Stop(CODEC_PDWN_SW); // stop CODEC
  clear_buffers();
  stm32_PLLI2S_MCLK_setup(sys_config[fastchip][mode].plln,
		                  sys_config[fastchip][mode].pllr); // set PLLI2S
  #endif // #if defined(STM32F4XX)
#endif // #if defined(USE_12BIT_DAC)

  MAXUNITS   = sys_config[fastchip][mode].polyphony; // polyphony = number of units
  SEQTICK1ms = sys_config[fastchip][mode].tick1ms;   // tick count for 1ms
  SEQTICK4ms = sys_config[fastchip][mode].tick4ms;   // tick count for 4ms
  oct_ofs    = sys_config[fastchip][mode].oct_ofs;   // octave offset
  note_ofs   = sys_config[fastchip][mode].note_ofs;  // f-number table offset
  memcpy(&FS_ASC, sys_config[fastchip][mode].fs_asc, 2);    // ASCII code for fs
#ifdef REVERB
  delay_mem_size = SEQTICK1ms * delay_ms; // calc delay momory size
  if (DELAYMEMSIZE < delay_mem_size) delay_mem_size = DELAYMEMSIZE; // clip if too big
#endif
  midi_mon_part = 0;  // reset MIDI monitor part
  MAXMIDICH  = MAXUNITS;                   // supported MIDI channels
#if defined(STM32F4XX)
  EVAL_AUDIO_Init(OUTPUT_DEVICE_BOTH, 83, sys_config[fastchip][mode].fs_Hz);
// start CS43L22 stereo audio DAC playing by DMA
  EVAL_AUDIO_Play(dac_dma_buf[0],
                  DMA_BUF_SIZE_ADJ*sizeof(dac_dma_buf));
#endif
} // void set_config()

void stm32_flash_setup( void )
{
// Embedded Flash Interface setup

//  FLASH->ACR = 0x0012; // prefetch buffer enable, wait cycle = 2

//  FLASH_ACR = 0x0012; // prefetch buffer enable, wait cycle = 2
} // void stm32_flash_setup()

void stm32_nvic_setup( void )
{
// NVIC (Nested Vector Interrupt Controller)

//  SCB_ExceptionTableOffset =  0x00003000; // set vector table offset
  
//  SCB->VTOR =  0x00003000; // set vector table offset
} // void stm32_nvic_setup()

void stm32_gpio_setup( void )
{
// GPIO setting
#if defined(STM32F4XX)
  GPIO_InitTypeDef GPIO_st;

  RCC->AHB1ENR |= RCC_AHB1Periph_GPIOA; // enable PA clocking
  RCC->AHB1ENR |= RCC_AHB1Periph_GPIOC; // enable PC clocking
  RCC->AHB1ENR |= RCC_AHB1Periph_GPIOD; // enable PD clocking

  GPIO_StructInit(&GPIO_st);         // 2 MHz speed, push-pull
  GPIO_st.GPIO_Mode = GPIO_Mode_AF;  // alternate function
  GPIO_st.GPIO_PuPd = GPIO_PuPd_UP;  // pull up

// MCU LCD I/F Tx setup
  GPIO_st.GPIO_Pin = (1 << LCD_TX_BIT);    // LCD Tx bit
  GPIO_Init(LCD_TX_PORT, &GPIO_st); // init LCD Tx bit
  GPIO_PinAFConfig(LCD_TX_PORT, LCD_TX_BIT, LCD_TX_AF);

// MCU LCD I/F Rx setup
  GPIO_st.GPIO_Pin = (1 << LCD_RX_BIT);    // LCD Rx bit
  GPIO_Init(LCD_RX_PORT, &GPIO_st); // init LCD Rx bit
  GPIO_PinAFConfig(LCD_RX_PORT, LCD_RX_BIT, LCD_RX_AF);

// MIDI I/F Tx setup
  GPIO_st.GPIO_Pin = (1 << MIDI_TX_BIT);    // MIDI Tx bit
  GPIO_Init(MIDI_TX_PORT, &GPIO_st); // init MIDI Tx bit
  GPIO_PinAFConfig(MIDI_TX_PORT, MIDI_TX_BIT, MIDI_TX_AF);

// MIDI I/F Rx setup
  GPIO_st.GPIO_Pin = (1 << MIDI_RX_BIT);    // MIDI Rx bit
  GPIO_Init(MIDI_RX_PORT, &GPIO_st); // init MIDI Rx bit
  GPIO_PinAFConfig(MIDI_RX_PORT, MIDI_RX_BIT, MIDI_RX_AF);

// MIDI bps SENSe port setup
  GPIO_st.GPIO_Pin  = (1 << MIDI_SENS_BIT); // MIDI bps SENSe bit
  GPIO_st.GPIO_Mode = GPIO_Mode_IN;  // input mode

  GPIO_Init(MIDI_SENS_PORT, &GPIO_st); // init MIDI SENSe bit

// ADC1_IN15 port (PC5) setup
  GPIO_st.GPIO_Pin  = (1 << ADC_IN_BIT); // ADC input bit
  GPIO_st.GPIO_PuPd = GPIO_PuPd_NOPULL;  // no pull up / down
  GPIO_st.GPIO_Mode = GPIO_Mode_AN;  // analog mode

  GPIO_Init(ADC_IN_PORT, &GPIO_st); // init ADC input port
#endif
#if (0)
  RCC->APB2ENR |= RCC_APB2Periph_GPIOA; // enable PA clocking
  RCC->APB2ENR |= RCC_APB2Periph_GPIOB; // enable PB clocking
  GPIOA->BSRR = (1<<10) | (1<<8); //  pullup PA10, PA8

#if defined(TGSTBM)        // for STBee Mini board
  RCC->APB2ENR |= RCC_APB2Periph_AFIO; // enable AFIO clocking
  AFIO->MAPR |= 0x02000000; // JTAG disabled, SWD enabled (PA15 free)
  GPIOA->CRH = 0x200008A8; // PA15 = GPIO out  p-p 2 MHz
                           // PA10 = USART1 RX with pullup
                           // PA9  = USART1 TX p-p 2 MHz
                           // PA8  = GPIO in   with pullup
#else
  GPIOA->CRH = 0x000008A8; // PA10 = USART1 RX with pullup
                           // PA9  = USART1 TX p-p 2 MHz
                           // PA8  = GPIO in   with pullup
#endif
  GPIOB->CRH = 0x9090000A; // PB8  = TIM4_CH3  p-p out  2 MHz
                           // PB13 = SPI2_SCK  p-p out 10 MHz
                           // PB15 = SPI2_MOSI p-p out 10 MHz
  GPIOB->CRL = 0x000000A0; // PB1  = TIM3_CH4  p-p out  2 MHz

// CQ-STARM board LED port (PC6)
#if defined(TGSTCQ)
  RCC->APB2ENR |= RCC_APB2Periph_GPIOC; // enable PC clocking
  GPIOC->CRL = 0x02008000; // PC6  = GPIO out  p-p out  2 MHz, 
                           // PC3  = GPIO in   with pullup
#endif

// STBee board LED port (PD4)
#if defined(TGSTBEE)
  RCC->APB2ENR |= RCC_APB2Periph_GPIOD; // enable PD clocking
  GPIOD->CRL = 0x88828888; // PD4  = GPIO out  p-p out  2 MHz, 
#endif
#endif
} // void stm32_gpio_setup()

void stm32_timer_setup( void )
{
// Timer 3 setup ( fs generation )

  RCC->APB1ENR |= RCC_APB1Periph_TIM3; // enable clock for TIM3
  RCC->APB1ENR |= RCC_APB1Periph_TIM4; // enable clock for TIM4
#if (defined(STM32F4XX))
  RCC->APB2ENR |= RCC_APB2Periph_TIM8; // enable clock for TIM8
  RCC->AHB1ENR |= RCC_AHB1Periph_GPIOC; // enable PC clocking
#endif

#if defined(USE_12BIT_DAC)
  TIM3->CR1   = 0x94;  // down count, interrupt on OVF/UDF only
  TIM3->CR2   = 0x00;  // 
  TIM3->SMCR  = 0x00;  // slave mode disable
  TIM3->CCMR1 = 0x00;  // disable ch1, ch2 compare
  TIM3->CCMR2 = 0x6800; // ch3 = disable, ch4 = PWM1 mode, OC4 = out

  TIM3->PSC   = 0;       // set prescaler to 1/1
  TIM3->ARR   = 4500-1;  // set sampling frequency to 16 kHz
  TIM3->CCR4  = (4500 >> 1);    // duty register

  TIM3->CCER  = 0x3000; // OC4 enable and invert
  TIM3->DIER  = 0x0001; // enable UIE only
//  NVIC->Enable[0] |= (1 << 29); // enable TIM3 interrupt   
  NVIC->ISER[0] |= (1 << 29); // enable TIM3 interrupt   
  TIM3->CR1   |= 0x01;  // enable timer 3

#if (defined(STM32F4XX))
  // Timer 4 setup ( Ch3 for 10 bit PWM DA )

  TIM4->PSC = 0;    // set prescaler to 1/1
  TIM4->ARR = (TIM3->ARR); // set PWM period

  TIM4->CR1 = 0x0090;  // down count
  TIM4->CR2 = TIM_TRGOSource_Update;  //

  TIM4->CCR3  = 1;    // duty register
  TIM4->CCMR1 = 0x0000; // disable ch1, ch2 compare
  TIM4->CCMR2 = 0x0068; // ch3 = PWM1 mode, OC3 = output, disable ch4
  TIM4->CCER  = 0x0100; // OC3 enable
  TIM4->SMCR  = 0x0000; // disable slave mode

  TIM4->CR1   |= 0x01;  // enable timer 4
#ifdef DEMOONLY
  TIM4->ARR = 1000-1; // 72 kHz rate
#endif

#if (0)
// Timer 8 setup ( TIM8_CH2 for CORDEC MCK (PC7) )
  TIM8->PSC = 0; // set prescaler to 1/1
  TIM8->ARR = 6; // 168 MHz / (7*2) = 12 MHz
  TIM8->CR1 = 0x0090;  // down count
  TIM8->CR2 = 0;  //

  TIM8->CCR2  = 0;      // Ch2 compare register
  TIM8->CCMR1 = 0x3800; // CH2 = master, toggle, OC2 = output
  TIM8->CCMR2 = 0x0000; // disable CH3, CH4 compare
  TIM8->CCER  = 0x0010; // OC2 enable
  TIM8->SMCR  = 0x0000; // disable slave mode
  TIM8->BDTR  = 0xC000; // MOE =1, AOE = 1

  TIM8->CR1   |= 0x01;  // enable TIM8

  GPIO_PinAFConfig(GPIOC, GPIO_PinSource7, GPIO_AF_TIM8);
#endif  
#else
// Timer 4 setup ( Ch3 for 10 bit PWM DA )

  TIM4->PSC = 0;    // set prescaler to 1/1
  TIM4->ARR = 1023; // set PWM period

  TIM4->CR1 = 0x0090;  // down count
  TIM4->CR2 = 0x0000;  // 

  TIM4->CCR3  = 512;    // duty register
  TIM4->CCMR1 = 0x0000; // disable ch1, ch2 compare
  TIM4->CCMR2 = 0x0068; // ch3 = PWM1 mode, OC3 = output, disable ch4
  TIM4->CCER  = 0x0100; // OC3 enable
  TIM4->SMCR  = 0x0000; // disable slave mode

  TIM4->CR1   |= 0x01;  // enable timer 4
#ifdef DEMOONLY
   TIM4->ARR = 1000-1; // 72 kHz rate
#endif
#endif
#endif
} // void stm32_timer_setup()

void stm32_spi_setup( void )
{
#if (0)
// SPI setup for 16-bit Serial DAC
  RCC->APB1ENR |= RCC_APB1Periph_SPI2; // enable clock for SPI2
  SPI2->CR1 = 0x0B44 + (0x03 << 3); //  16bit,master,MSB first,72/16=4.5MHz clk
#endif
} // void stm32_spi_setup()

void stm32_usart_setup( void )
{
#if defined(STM32F4XX)
// USART setup

	  RCC->APB1ENR |= RCC_APB1Periph_USART2;  // enable clock for USART2 (42 MHz)
	  RCC->APB1ENR |= RCC_APB1Periph_USART3;  // enable clock for USART3 (42 MHz)

// USART for MCU LCD I/F setup for LCD MCU interface (62.5 kbps)

      LCD_RX_UART->BRR  = 672;     // 42 MHz /  62.5 kbs = 672; ( 0 %)
	  LCD_RX_UART->CR1  = 0x2000;  // USART enable, 8-bit data, Non parity, no interrupt
	  LCD_RX_UART->CR2  = 0x0000;  // 1 stop bit
	  LCD_RX_UART->CR3  = 0x0000;  // no RTS/CTS, no flow control, no DMA
	  LCD_RX_UART->CR1 |= 0x0C;    // RX, TX enable

	  serial_midi = (0 != ((1 << MIDI_SENS_BIT) & MIDI_SENS_PORT->IDR));

	  if (serial_midi) {     // serial MIDI (38.4 kbps)
	    MIDI_RX_UART->BRR  = 1094; // 42 MHz /  38.4  kbps = 1094; (-0.023 %)
	  } else {               // normal MIDI (31.25 kbps)
	    MIDI_RX_UART->BRR  = 1344; // 42 MHz /  31.25 kbps = 1344; ( 0 %)
	  } // if (serial_midi)

	  MIDI_RX_UART->CR1  = 0x2000;  // USRT3 enable, 8-bit data, Non parity, no interrupt
	  MIDI_RX_UART->CR2  = 0x0000;  // 1 stop bit
	  MIDI_RX_UART->CR3  = 0x0000;  // no RTS/CTS, no flow control, no DMA
	  MIDI_RX_UART->CR1 |= 0x0C;    // RX, TX enable
#else
// USART1 setup

  RCC->APB2ENR |= RCC_APB2Periph_USART1;  // enable clock for USART1
 
  serial_midi = (0 != ((1<<8) & (GPIOA->IDR))); // test for PA8

  if (serial_midi) {     // serial MIDI (38.4 kbps)      
    USART1->BRR  = 1875; // 72 MHz /  38.4 kbps = 1875;
  } else {               // normal MIDI (31.25 kbps)
    USART1->BRR  = 2304; // 72 MHz /  31.25 kbps = 2304;
  } // if (serial_midi)
  
  USART1->CR1  = 0x2000;  // USRT1 enable, set 8-bit data, Non parity, no interrupt
  USART1->CR2  = 0x0000;  // set 1 stop bit
  USART1->CR3  = 0x0000;  // no RTS/CTS, no flow controll, no DMA
  USART1->CR1 |= 0x0C;    // RX, TX enable
#endif
} // void stm32_usart_setup()


void stm32_clock_setup( void )
{
#if (0)
// Clock configuration setup 

#if ((12000000UL) == HSE_Value) // 12 MHz HSE Xtal used
  RCC->CFGR = 0x0001C402     // PLLin=HSE, ADC/8, APB2=HCLK, APB1=HCLK/2, AHB=SYSCLK, SYSCLK=PLL
            | RCC_PLLMul_6 ; // (12 MHz Xtal) x (PLLx6) = 72 MHz
#else                        // 8 MHz HSE Xtal used
  RCC->CFGR = 0x0001C402     // PLLin=HSE, ADC/8, APB2=HCLK, APB1=HCLK/2, AHB=SYSCLK, SYSCLK=PLL
            | RCC_PLLMul_9 ; // ( 8 MHz Xtal) x (PLLx9) = 72 MHz
#endif

  RCC->CR   = 0x01010001;  // PLL ON, HSE ON, HSI ON
  while (((1<<25) & (RCC->CR)) == 0);  // wait for PLL ready
#endif
#if defined(STM32F4XX)
  stm32_PLLI2S_MCLK_setup(258, 3); // MCLK = 86 MHz
#endif
} // void stm32_clock_setup()

void stm32_dac_setup( void )
{
//   GPIO_InitTypeDef GPIO_InitStruct;
//	  DAC_InitTypeDef  DAC_InitStruct;

  RCC->APB1ENR |= RCC_APB1Periph_DAC; // enable clock for DAC

  DAC->CR &= ~(DAC_CR_EN1); // diablse DAC1
  DAC->DHR12R1 = 0;
// DAC1 enable, trigger = TIM3_TRGO, Buffer off (for linearity)
  DAC->CR = ( 0
		     | DAC_Trigger_T4_TRGO        // Trigger by TIM4_TRGO
#if (0)
		     | DAC_Wave_Triangle          // Triangle wave generation
		     | DAC_TriangleAmplitude_4095 // Amplitude = 12 bit
#endif
		     | DAC_CR_EN1                 // DAC1 enable
#if (0  & !defined(USE_DACBUF)) // defeat DAC buffer amp
		     | DAC_CR_BOFF1
#endif
		    );
} // void stm32_dac_setup()

void stm32_adc_setup(void)
{
  ADC_InitTypeDef ADC_st;
  ADC_CommonInitTypeDef ADC_Comm_st;
// enable ADC1 clock (84 MHz)
  RCC_APB2PeriphClockCmd(ADC_RCC, ENABLE);
// initialize common structure
  ADC_CommonStructInit(&ADC_Comm_st);
// ADC clock = APB2 clock (82 MHz) / 8 = 10.5 MHz
  ADC_Comm_st.ADC_Prescaler = ADC_Prescaler_Div8;
  ADC_CommonInit(&ADC_Comm_st);
// initialize ADC1 individual parameter
  ADC_StructInit(&ADC_st); // default settings
                           // 12 bit, right justify
  ADC_Init(ADC_UNIT, &ADC_st);
// sample time = 480 / 10.5 [MHz] = 45.7 [us]
  ADC_RegularChannelConfig(ADC_UNIT, ADC_CH, 1, ADC_SampleTime_480Cycles);

  ADC_Cmd(ADC_UNIT, ENABLE);
} // void stm32_adc_setup()

void stm32_port_setup(void)
{
  stm32_flash_setup();
  stm32_clock_setup();
  stm32_nvic_setup();
  stm32_gpio_setup();
  stm32_timer_setup();
  stm32_spi_setup();  
  stm32_usart_setup();
  stm32_adc_setup();
#if defined(USE_12BIT_DAC)
  stm32_dac_setup();
#endif
//  fastchip = speed_test(); // test chip speed;
  // select built-in 12 bit DAC module for audio output

  #if defined(USE_12BIT_DAC)
    EVAL_AUDIO_SetAudioInterface(AUDIO_INTERFACE_DAC);
  #else
    EVAL_AUDIO_SetAudioInterface(AUDIO_INTERFACE_I2S);
  #endif

  // activate external stereo DAC (CIRRUS LOGIC CS43L22)
  // for analog input (AIN1A,B) passthru
  // to headphone output (HP/LINE_OUTA,B)

    EVAL_AUDIO_Init(OUTPUT_DEVICE_BOTH, 83, I2S_AudioFreq_48k);
  // start CS43L22 stereo audio DAC playing by DMA
    EVAL_AUDIO_Play(dac_dma_buf[0],
                    DMA_BUF_SIZE_ADJ*sizeof(dac_dma_buf));
} // void stm32_port_setup()

#if (0)
static inline void __enable_irq()
{ __asm volatile ("cpsie i"); 
}
#endif

void enable_interrupt(void)
{
  __enable_irq();
} // void enable_interrupt()
#endif


void fill_dma_buf(int n)
{
// copy audio sample data from wave_que[] to DMA buffer
  memcpy(dac_dma_buf[n], wave_que[wq_rd_ix], sizeof(dac_dma_buf[0]));
  UPDATE_QUE_READ_IX(AUDIO_SAMPLES_PER_DMA_BUF);
  t1_tick = 1;
} // void fill_dma_buf()

// Callback used by stm32f4_discovery_audio_codec.c.
// Refer to stm32f4_discovery_audio_codec.h for more info.

void EVAL_AUDIO_TransferComplete_CallBack(uint32_t pBuffer, uint32_t Size){
// "1"-page of DMA buffer has transfered, fill it
  fill_dma_buf(1);
}

void EVAL_AUDIO_HalfTransfer_CallBack(uint32_t pBuffer, uint32_t Size){
// "0"-page of DMA buffer has transfered, fill it
  fill_dma_buf(0);
}

/*
 * Callback used by stm324xg_eval_audio_codec.c.
 * Refer to stm324xg_eval_audio_codec.h for more info.
 */
uint16_t EVAL_AUDIO_GetSampleCallBack(void){
  /* TODO, implement your code here */
  return -1;
}

// place holder to avoid compile error
#if (0)
uint32_t Codec_TIMEOUT_UserCallback(void)
{
  /* Block communication and all processes */
  while (1)
  {
  }
} // uint32_t Codec_TIMEOUT_UserCALLback()
#endif
