/**************************************************/
/*                                                */
/* midifunc.c : various MIDI functions            */
/*                                                */
/* NON COMMERCIAL USE ONLY                        */
/*                                                */
/* 2011/04/29 Rev. 1.2  for SMF sequencer         */
/* 2011/04/12 Rev. 1.02 S-MIDI F5h multi port     */
/* 2007/06/26 Rev. 1.01 by pcm1723                */
/*                                                */
/**************************************************/

#include "tonegen.h"

/* MIDI note off */
void midi_note_off(ch_prop_t *c_p, int md_note)
{
  unit_prop_t *u_p = &(unit_prop[c_p->in_use[0]]);
  if (0 > md_note) { // force note off
    u_p->on_cnt = 0;
    u_p->gate   = 0;
    set_egmode(u_p, FORCE_DAMP);
#ifdef LCD
    if ((MIDI_MON == panel_mode) & (midi_mon_part == c_p->midi_ch)){ //
      int i;
      for (i = 0; i < 12; i++)
        lcd_buf[MON_KEY_DISP_OFS+i] &= 0x7e; // key off bitmap
        lcd_buf[MON_OCT_DISP_OFS]  = ' ';
        lcd_buf[MON_SW_DISP_OFS]   = '_';
        lcd_buf[MON_NN_DISP_OFS]   = ' ';
        lcd_buf[MON_NN_DISP_OFS+1] = ' ';
        lcd_buf[MON_VEL_DISP_OFS]  = ' ';
        lcd_buf[MON_VEL_DISP_OFS+1]= ' ';
    } // if (MIDI_NOTE == panel_mode) ...
#endif
#if syn_CV
    if (syn_CV_ch == u_p->midi_ch) {
    GATE_OFF;
//      GP2DAT &= 0xfff7ffff ; // GATE off
//        GP2DAT |= 0x00080000 ; // GATE on
    }
#endif 
  }
  else { // normal note off
    u_p->on_cnt--;
    if (0 >= u_p->on_cnt) { // the only one note off
      u_p->porta_flag  = 0; // portamento mode off
      u_p->porta_delta = 0;
      u_p->on_cnt = 0; // reset keyon count
#if syn_CV
      if (syn_CV_ch == u_p->midi_ch) {
      GATE_OFF;
//        GP2DAT &= 0xfff7ffff ; // GATE off
//        GP2DAT |= 0x00080000 ; // GATE on
    }
#endif 
      u_p->gate   = 0;
#if defined(SEQUENCER)      
      if (demoflag) {
        u_p->op_prop[0].gate_q <<= 1;  // shift last gate state
        u_p->op_prop[1].gate_q <<= 1;  // shift last gate state
      } // if 
#endif      
    } // if (0 => u_p->on_cnt)
#ifdef LCD
    if ((MIDI_MON == panel_mode) & (midi_mon_part == c_p->midi_ch)){ //
      char c;
      switch (u_p->on_cnt) { // how many keys left ?
        case 0  : c = '_'; break; // no key pressed
        case 1  : c = ':'; break; // 1 key pressed
        default : c = '#'; break; // 2 or more
      } // switch
      lcd_buf[MON_SW_DISP_OFS] = c;
//      lcd_buf[MON_OCT_DISP_OFS+1] = c;
    } // if (MIDI_NOTE == panel_mode) ...
#endif
    c_p->last_off_note = md_note;
  } // if (0 < md_note) else ...
} /* void midi_note_off() */

/* MIDI note on */
void midi_note_on(ch_prop_t *c_p, int md_note, int md_vel)
{
  int p_src, k_src, o_src;
  unit_prop_t *u_p = &(unit_prop[c_p->in_use[0]]);
  p_src            = c_p->MD_porta_src; // portamento source note
  c_p->MD_note     = md_note;
  c_p->last_on_note= md_note;
  c_p->MD_velocity = md_vel;
  u_p->midi_note   = md_note;
  u_p->porta_flag  = (0x40 <= c_p->MD_porta_sw);
  u_p->porta_flag &= (0 != c_p->MD_porta_time); // clear porta mode if time==0
  u_p->porta_flag &= (0 != (md_note - p_src)); // clear porta mode if same note
  k_src = 0x0f & octkey[p_src] ; // get keycode of source note
  u_p->note_f   = (p_src << 16) // portamento source note number
                + ((int)c_p->so_tune_ofs[k_src] << 9); 
  k_src = 0x0f & octkey[md_note] ;  // get keycode of target note
  o_src = octkey[md_note] >> 4;  // get octavecode of target note
  u_p->m_note_f   = (md_note << 16) // portamento target note number
                  + ((int)c_p->so_tune_ofs[k_src] << 9); 
  if (u_p->porta_flag) { // portamento switch ON
    if (md_note > p_src) { // ascending
      u_p->porta_delta =  c_p->porta_inc; // increment amount
    }
    else { // descending
      u_p->porta_delta = -c_p->porta_inc; // decrement amount
    } // if (md_note ...
  }  // if (u_p->porta_flag) ...
  else { // no portamento mode
//    u_p->note_f = md_note << 16; // fractional MIDI note number 
    p_src            = md_note;       // source note = target note
    u_p->note_f      = u_p->m_note_f; // source note fraction = target note fraction
    u_p->porta_delta = 0;             // update rate = 0 
  } // else
  u_p->midi_vel = c_p->MD_velocity;
  u_p->vel_lb   = mv2lb[c_p->MD_velocity];
  u_p->exp_lb   = (c_p->exp_lb + c_p->vol_lb + master_prop.master_vol_lb);
  c_p->MD_porta_src= md_note;
  calc_pitch(c_p, u_p);
  calc_keyscaling(u_p);
#if syn_CV
    if (syn_CV_ch == u_p->midi_ch) {
    GATE_ON;
//      GP2DAT |= 0x00080000 ; // GATE on
//      GP2DAT &= 0xfff7ffff ; // GATE off
    }
#endif 
  if (1 < (++(u_p->on_cnt))) { // increment keyon count
    if (64 > c_p->MD_legato) { // 2 or more keyon, non legato, start EG
      unit_prop_t *u_p = &(unit_prop[c_p->in_use[0]]);
      u_p->op_prop[0].gate_q &= 0xfe; // force gate-off
      u_p->op_prop[1].gate_q &= 0xfe; // force gete-off
    } // if (64 >
  } // if (++u_p->on_cnt)
  u_p->gate = 1; // gate ON
#ifdef LCD
  if ((MIDI_MON == panel_mode) & (midi_mon_part == c_p->midi_ch)){ //
    if (0 <= last_note ) { // last note present, erase it
      lcd_buf[MON_KEY_DISP_OFS + last_note] &= 0x7e; // set b0 = 0
    } // if (0 <= last_note)
    lcd_buf[MON_KEY_DISP_OFS + k_src] |= 0x01; // set b0 = 1
//    lcd_buf[MON_OCT_DISP_OFS+1] = ((1 < u_p->on_cnt) ? '#' : ':');
    lcd_buf[MON_OCT_DISP_OFS] = hex_str[o_src];
    lcd_buf[MON_SW_DISP_OFS]   = ((1 < u_p->on_cnt) ? '#' : ':');
    lcd_buf[MON_NN_DISP_OFS]   = hex_str[0x0f & (md_note >> 4)];
    lcd_buf[MON_NN_DISP_OFS+1] = hex_str[0x0f &  md_note      ];
    lcd_buf[MON_VEL_DISP_OFS]  = hex_str[0x0f & (md_vel >> 4)];
    lcd_buf[MON_VEL_DISP_OFS+1]= hex_str[0x0f &  md_vel      ];
    last_note = k_src;
  } // if (MIDI_NOTE == panel_mode) ...
#endif
} /* void midi_note_on() */

/* update portamento */

void update_porta(unit_prop_t *u_p)
{
  long m_note_f;
  if (u_p->porta_flag) { // portamento enabled
    u_p->note_f += u_p->porta_delta; // update fractional note number
    m_note_f = u_p->m_note_f;        // target note with fraction
    if (0 > u_p->porta_delta) {  // descending
      if (m_note_f >= u_p->note_f) { // reached to target
        u_p->note_f = m_note_f; // replace by target note
        u_p->porta_flag = 0;
        u_p->porta_delta = 0;
      } // if (m_note_f
    }
    else { // ascending
      if (m_note_f <= u_p->note_f) { // reached to target
        u_p->note_f = m_note_f; // replace by target note
        u_p->porta_flag = 0;
        u_p->porta_delta = 0;
      } // if (m_note_f
    } // if (0 > ...
//    calc_pitch(&(ch_prop[u_p->midi_ch]), u_p);
  } // if (u_p->porta_flag)
} // void update_porta()

/* Calculate portamento time constant */

void calc_porta_time(ch_prop_t *c_p, int ptime)
{
  c_p->porta_inc = portab[ptime];  // use lower 4-bit as table index
#if debug
/*
  lcd_gotoxy(0, 0);
  printf("pinc=%8.8x ", c_p->porta_inc);
*/
#endif
} // void calc_porta_time()

/* Calculate log2 value of MIDI ch volume, MIDI expression  */
void calc_vol_exp(ch_prop_t *c_p)
{
  int i;
  unit_prop_t *u_p = &(unit_prop[c_p->in_use[0]]);
  master_prop.master_vol_lb = mv2lb[master_prop.MD_master_volume]; 
  c_p->exp_lb = mv2lb[c_p->MD_expression];
  c_p->vol_lb = mv2lb[c_p->MD_volume];
  u_p->exp_lb = (c_p->exp_lb + c_p->vol_lb + master_prop.master_vol_lb);
  for (i = 0; i < 2; i++) { // for each operator
    op_prop_t *o_p = &(u_p->op_prop[i]);
    if (o_p->vp.carrf) { // if operator is carrier
      o_p->vol_lb = (u_p->exp_lb << 16);
    } // if (o_p->carff) ...
    else o_p->vol_lb = 0;
  } // for (i=0; i < 2; i++) ...
/*
if (2 > u_p->midi_ch) {
lcd_gotoxy(0, u_p->midi_ch);
printf("v:e=%2.2x:%2.2x ",  c_p->MD_volume, c_p->MD_expression);
} // if
*/
} // void calc_vol_exp()

/* calculate EG rate key scaling and output level key scaling */
void calc_keyscaling(unit_prop_t *u_p)
{
  int targnote, oct, k, i, s;
  short int w0, ksn;
  targnote = u_p->midi_note;
  oct = octkey[targnote]; 
  k   = (0x0f & oct);     // extract note number in octave
  oct >>= 4;              // extract octave part
  ksn = ((oct - 1) << 1) + (6 <= k); // key scaling number
  if (0  > ksn) ksn = 0;              // replace by 0 if negative
  if (15 < ksn) ksn = 15;             // replace by 15 if too high
  w0 = ((oct - 3) << 6) + ks60tab[k]; // convert oct:key to ksl 6dB/oct value
  if (0 > w0)  w0 = 0;                // replace by 0 if negative
  u_p->ksn     = ksn;
  u_p->ks6_0dB = w0; 
/*
if (0 == u_p->midi_ch) {
lcd_dbg_hex2(0, targnote);
lcd_buf2[2] = ' ';
lcd_dbg_hex4(3, w0);
lcd_buf2[7] = ' ';
lcd_dbg_hex2(8, ksn);
lcd_buf2[10] = ' ';
}
*/
  for (i = 0; i < 2; i++) { // set EG rate key scaling for each operator
    op_prop_t *o_p = &(u_p->op_prop[i]); // operator property pointer
    if (o_p->vp.ksr) o_p->rof =  ksn;      // KSR = 1
    else             o_p->rof = (ksn >>2); // KSR = 0
    s  = o_p->vp.tl; // Total Level in log2
    switch (o_p->vp.ksl) {
      case 1 : 
        s += ((u_p->ks6_0dB)>>1) ; //   3dB/oct
        break;
      case 2 : 
        s += ((u_p->ks6_0dB)>>2) ; // 1.5dB/oct
        break;
      case 3 : 
        s += ((u_p->ks6_0dB)   ) ; //   6dB/oct
        break;
      default: 
        break;
    } // switch (o_p->ksl)
    if (o_p->vp.carrf) { // is this op a carrier ?
      s += u_p->vel_lb; // apply velocity to total level
    } // if (o_p->carrf) ...
    else o_p->vol_lb = 0;
    o_p->tl_lb  = (s << 16); // TL + KSL ( + Vel)
  } // for (k = i; i < 2; i++) 
/*
lcd_gotoxy(0, 0);
printf("ks6_0dB=%2.2x, %4.4lx ",  w0, u_p->ks6_0dB);
*/
} // void calc_keyscaling()

/* Process pedal switches */
void pedal_sw(ch_prop_t *c_p, int pedal, int val)
{
  unit_prop_t *u_p = &(unit_prop[c_p->in_use[0]]);
  switch (pedal) { //
    case 0x40 : // CC#64 HOLD1/DAMPER
      c_p->MD_hold1 = val;
      u_p->damper = (0x40 <= val);
      break;
    default: break;
  } // switch (pedal)
} // void pedal_sw()

/* calculate AM (tremolo)  */
void calc_am(ch_prop_t *c_p, unit_prop_t *u_p)
{
  int i;
  for (i = 0; i < 2; i++) { // for each operator
    op_prop_t *o_p = &(u_p->op_prop[i]);
    if (o_p->vp.am) { // AM enabled
      o_p->am_lb = ((uint32_t)c_p->aLFO << (o_p->vp.dam + 13)); // 1.5dB for DAM=0, 12dB for DAM=3
    } // if (o_p->am)
    else o_p->am_lb = 0; // no AM
/*
if (0 == u_p->midi_ch) {
lcd_gotoxy(0, u_p->midi_ch);
printf("alfo=%8.8x ",  o_p->am_lb);
}
*/
  } // for (i = 0; i < 2; i++) ...
} // void calc_am()

/* calculate pitch from MIDI note with portament, pitch bend, */
/* tune and LFO vibrato for each operator                     */
void calc_pitch(ch_prop_t *c_p, unit_prop_t *u_p)
{
  uint32_t ph_inc;
  int targnote, oct, oct0, k, kk, kk0, fine, i;
  for (i = 0; i < 2; i++) { // for each operator
    op_prop_t *o_p = &(u_p->op_prop[i]); // operator property pointer
// apply MIDI CC#1 "MODULATION" and MIDI CC#77 "VIBRATO DEPTH" to DVB of voice data
    k = o_p->vp.dvb + (c_p->MD_modulation >> 5) + (c_p->MD_vib_depth >> 4); 
    if (0 > k) k = 0; // too low, limit to 0
    if (3 < k) k = 3; // too high, limit to 3
    if ((0 == c_p->MD_modulation) | (0 == o_p->vp.vib)) { //
      targnote = 0; // no vibrato
    }
    else targnote  = (c_p->vLFO_f >> (4 - k));     // apply vibrato 
    targnote += u_p->note_f + c_p->bend_f + c_p->tune_f + master_prop.master_tune_f;
    while (0          >  targnote) {targnote += 0xc0000;} // octave up if note too low
    while (0x00800000 <= targnote) {targnote -= 0xc0000;} // octave down if note too high
    oct = octkey[targnote >> 16]; 
    k   = (0x0f & oct);     // extract note number in octave
    oct >>= 4;              // extract octave part
    oct0 = oct;             // remember original octave for synCV
    fine = ((0xfe00 & targnote) >> 9); // extract fraction part
    kk = (k << 7) + fine + note_ofs; // combine note with fraction
    kk0 = kk - note_ofs;
    while (FNUMTAB_MAX <= kk) { // index falls outside of fnum table
        kk -= FNUMTAB_MAX; // reduce fnum table index
        oct++;             // increment octave code
      }
    ph_inc = fnum128[kk] << (oct + oct_ofs);
    o_p->ph_inc = ph_inc * o_p->vp.mult; // multiply by operator freq setting

//lcd_dbg_hex8(0, o_p->ph_inc); lcd_dbg_hex4(9, o_p->vp.mult);

    if (1 == i) { // carrier
      u_p->oct     = oct;
      u_p->keycode = k;
      u_p->ph_inc  = ph_inc;
#if syn_CV
      u_p->linCV_oct  = oct;
      u_p->linCV_mant = fnum128[kk];
      if (syn_CV_ch == u_p->midi_ch) {
        short pdmw, oct1;
        pdmw = kk0 + syn_CV_ofs;
        oct1 = oct0;
        if (FNUMTAB_MAX <= pdmw) {
          pdmw -= FNUMTAB_MAX;
          oct1++;
        } // if
        pdmw   = fnum128[FNUMTAB_MAX-1 - pdmw];   // dnumber
        pdmw  &= 0xfff0;                          // extract MS 12 bits
        pdmw   = -pdmw;                           // 2's complement
        PUT_SPIQ( 0xff & (pdmw >> 8));            // dnumber upper 8 bit
        PUT_SPIQ((0xf0 &  pdmw) | (0x0f & oct1)); // dnumber lower 4 bit + octave_code

      } // if (0 == u_p->midi_ch)
#endif
    } // if (0 == i)
#if debug
/*
lcd_gotoxy(0, 0);
printf("N%2.2x n%1.1x f%1.1x.%1.1x:%2.2x ", 
       u_p->midi_note, u_p->ksn,
       u_p->oct,       u_p->keycode,
       u_p->frac);
lcd_gotoxy(0, 1);
printf("i:%8.8lx ",  u_p->ph_inc);
*/
#endif
  } // for (k = i; i < 2; i++) 
} // void calc_pitch()

/* All note off */
void all_note_off(ch_prop_t *c_p)
{
//  unit_prop_t *up = &(c_p->in_use[0]);
  midi_note_off(c_p, -FORCE_DAMP);
} // void all_note_off();

/* GM system on */
void GM_system_on(void)
{
  int i, ch;
  master_prop_t *m_p = &master_prop;
  m_p->MD_master_volume = 127; // Master volume
  for (ch = 0; ch < MAXMIDICH; ch++) {
    ch_prop_t *cp = &(ch_prop[ch]);
    cp->in_use[0] = ch;
    all_note_off(cp);
    reset_all_controllers(cp);
    cp->MD_rev_send    =  64; // MIDI reverb send level = 64
#if defined(CHORUS)    
    cp->MD_chorus_send =  64; // MIDI chorus send level = 0
#endif
    cp->MD_volume      = 100; // MIDI volume      level = 100
    cp->MD_coarse_tune =  64; // MIDI coarse tune = 0
    cp->MD_fine_tune   =  64; // MIDI fine   tune = 0
    cp->MD_pan         =  64; // MIDI pan = center
    cp->MD_porta_time  =   0; // MIDI portamento time = 0
    cp->MD_bend_range  = 2;      // MIDI pitch bend range = 2 semitone
    cp->MD_mod_range   = 0x0080; // MIDI modulation range = 50 cent
    cp->fine_tune      = 0;
    cp->coarse_tune    = 0;
    MD_port_mask       = 0x00;
    for (i = 0; i < 12; i++) cp->so_tune_ofs[i] = 0; // reset micro tuning
#ifdef LCD
//    lcd_disp_timer = 0;
//    lcd_buf_p = lcd_buf;
#endif
    calc_vol_exp(cp);
    setup_voice(cp, 0);  
    copy_voice(&(unit_prop[ch]));
  } // for (ch
} // void GM_system_on()

/* Hard reset */
void hard_reset(void)
{
  int i, k, ch;
  for (k = 0; k < MAXUNITS; k++) {
    int m;
    unit_prop_t *u_p = &(unit_prop[k]);
    ch_prop[k].midi_ch = k;
    u_p->midi_ch = k;   // remember assigned channel
    for (m = 0; m < 2; m++) {
      op_prop_t *o_p = &(u_p->op_prop[m]);
    } // for (m
  } // for (k =
  MD_bcnt = 0; // MIDI message byte count 
  MS_run_stat = 0x00; // MIDI running status 
#if (defined(SEQUENCER) & !defined(USE_SMF))
  for (ch = 0; ch < MAXSEQCH; ch++) {
    seq_st[ch].gatetime = -1;
  } // for
#endif
  for (i = 0; i < ABS_MAX_PARTS; i++) ch_to_part[i] = i; // reset MIDI channel map
  for (i = 0; i < ABS_MAX_PARTS; i++) part_to_ch[i] = i; // reset MIDI channel map
  if ((MIDI_DRUM_CH <= MAXUNITS) & (15 >= MAXUNITS)) { // 10 <= UNITS <= 15
// remap part-to-ch assign to skip drum ch (10)  
    int ch;
    for (ch = (MIDI_DRUM_CH-1); ch < MAXMIDICH; ch++) {
      part_to_ch[ch]   = ch + 1;
      ch_to_part[ch+1] = ch;
    } // for
    ch_to_part[MIDI_DRUM_CH-1] = MAXMIDICH;
  } // if
  
  mon_ch = 0; // MIDI monitor channel
  syn_CV_ch = 0; // synthesizer CV channel
  master_prop.MD_master_volume      = 127;
  master_prop.MD_master_coarse_tune = 64;
  master_prop.MD_master_fine_tune   = 64;
  tick1ms = SEQTICK1ms;
  debounce = DEBOUNCE_ms; // sw input debounce timer
  GM_system_on();
} // void hard_reset()

/* Reset all controllers */
void reset_all_controllers(ch_prop_t *cp)
{
  cp->MD_expression  = 127;    // MIDI expression level = 127
  cp->MD_bend        = 0;      // MIDI pitch bend       = 0 (center)
  cp->MD_modulation  = 0;      // MIDI modulation       = 0
  cp->MD_vib_rate    = 0;      // MIDI vibrato rate     = 0
  cp->MD_vib_depth   = 0;      // MIDI vibrato depth    = 0
  cp->MD_vib_delay   = 0;      // MIDI vibrato delay    = 0
  cp->MD_RPN_ML      = 0x7f7f; // MIDI  RPN MSB/LSB     = 7fh (null)
  cp->MD_NRPN_ML     = 0x7f7f; // MIDI NRPN MSB/LSB     = 7fh (null)
  cp->MD_porta_sw    = 0;      // MIDI portamento switch= 0 (off)
  cp->MD_porta_src   = 0;      // MIDI portamento source
  cp->MD_legato      = 0;      // MIDI legato switch    = 0 (non legato)
  pedal_sw(cp, 64, 0);         // MIDI CC#64 hold1/damper = 0
  calc_tune_f(cp);
  pitch_bend(cp, 0);
} // void reset_all_controllers()

/* LFO update */
void update_LFO(ch_prop_t *c_p)
{
  long int vLFO_wave, aLFO_wave; // triangle LFO wave
  vLFO_wave = 0xffff & (c_p->LFO_ph_acc + c_p->LFO_ph_inc); // increment phase
  c_p->LFO_ph_acc = vLFO_wave; // save phase
  aLFO_wave       = vLFO_wave; // LFO for AM
// shape triangle wave from sawtooth
  vLFO_wave -= 0x8000; // shift to [-32768..32767]
  if ( 0x4000 <= vLFO_wave) vLFO_wave = ( 0x8000 - vLFO_wave);
  if (-0x4000 >  vLFO_wave) vLFO_wave = (-0x8000 - vLFO_wave);
  if ( 0x8000 <= aLFO_wave) aLFO_wave = ( 0xffff ^ aLFO_wave); // [0..0x7fff]
  c_p->vLFO_f = (vLFO_wave * c_p->MD_mod_range) >> 6; // apply modulation range
  c_p->aLFO   =  (aLFO_wave >> 8); // for -12dB range [0..127]
  calc_pitch(c_p, &unit_prop[c_p->in_use[0]]);
  calc_am   (c_p, &unit_prop[c_p->in_use[0]]);
} // void update_LFO()

/* calculate vibrato setting */

void calc_vibrato(ch_prop_t *c_p)
{
  int val;
  val = c_p->LFO_rate + (c_p->MD_vib_rate >> 4); // apply CC#76 MIDI vibrato rate change
  if (0 > val) val = 0; // too low
  if (3 < val) val = 3; // too high
  c_p->LFO_ph_inc = LFO_inc_tab[val]; // set LFO phase accumulator increment
} /* void calc_vibrato() */

/* calculate tunning value */

void calc_tune_f(ch_prop_t *c_p)
{
  c_p->tune_f = (c_p->coarse_tune << 16) + (c_p->fine_tune << 8);
  master_prop.master_tune_f 
    = ((master_prop.MD_master_coarse_tune - 0x40) << 16) 
    + ((master_prop.MD_master_fine_tune   - 0x40) << 10);
} /* void calc_tune_f() */

/* pitch bend */
void pitch_bend(ch_prop_t *c_p, int32_t bend)
{
  unit_prop_t *u_p = &(unit_prop[c_p->in_use[0]]);
  bend *= c_p->MD_bend_range; // mult by bend range (1..24)
  bend <<= 3; // b31..b16 <- int part, b15..b3 <- fraction
  c_p->bend_f = bend;
  calc_pitch(c_p, u_p);
} // void pitch_bend()

/* pre load voice data to channel property */
void preload_voice(ch_prop_t *c_p, int pc)
{ 
  c_p->vdat = *((struct voicedata *)&vma2[pc]);
} // void preload_voice()

/* decode voice data from MA-2 format data */
void decode_voice(ch_prop_t *c)
{ 
  int k; 
  unit_vparam_t *uvp = &(c->uv);
  struct voicedata *v = &(c->vdat);
  int8_t uno = c->in_use[0];  
  unit_prop_t *u_p = &(unit_prop[uno]);
  
  uvp->fb      = v->FB;
  uvp->lfo     = v->LFO;
  uvp->alg     = v->ALG;
/*
  unit_prop_t *up = &(c->patch);
  struct voicedata *v = &(c->vdat);
  up->fb      = v->FB;
  up->lfo     = v->LFO;
  up->alg     = v->ALG;
  up->on_cnt = 0;
  up->midi_ch = c->midi_ch;
*/
  for (k = 0; k < 2; k++) { 
    struct ma2reg *ma2 = &(v->op[k]);
//    op_prop_t *p = &(up->op_prop[k]);
    op_vparam_t *d = &(c->ov[k]);
    d->mult   = OPL3MULT (ma2->MULT);
    d->ksl    = ma2->KSL;
    d->tl     = OPL3LEVEL(ma2->TL);
    d->ar     = OPL3RATE (ma2->AR);
    d->dr     = OPL3RATE (ma2->DR);
    d->sl     = OPL3LEVEL((ma2->SL)<<2);
    d->rr     = OPL3RATE (ma2->RR);
    d->wavesel= ma2->WS;
    d->ksr    = ma2->KSR;
    d->egt    = ma2->EGT;
    d->am     = ma2->AM;
    d->dam    = ma2->DAM;
    d->vib    = ma2->VIB;
    d->dvb    = ma2->DVB;
    d->carrf  = k | (1 == uvp->alg);
    d->op_num = k; // operator number
/*
    p->mult   = OPL3MULT (ma2->MULT);
    p->ksl    = ma2->KSL;
    p->tl     = OPL3LEVEL(ma2->TL);
    p->ar     = OPL3RATE (ma2->AR);
    p->dr     = OPL3RATE (ma2->DR);
    p->sl     = OPL3LEVEL((ma2->SL)<<2);
    p->rr     = OPL3RATE (ma2->RR);
    p->wavesel= ma2->WS;
    p->ksr    = ma2->KSR;
    p->egt    = ma2->EGT;
    p->am     = ma2->AM;
    p->dam    = ma2->DAM;
    p->vib    = ma2->VIB;
    p->dvb    = ma2->DVB;
    p->carrf  = k | (1 == up->alg);
    p->egrate = 0;
    p->egmode = 0;
    p->ol_lin = 0;
    p->phacc  = 0;
    p->egacc  = 0;
    p->op_num = k; // operator number
    p->op_func_p = op_ws_p[ma2->WS]; // select function by WS
*/
    d->op_func_p = (int16_t (*)(void *)) op_ws_p[d->wavesel];
    u_p->voice_param_change = 1; // voice parameter change flag
    u_p->LFO_param_change   = 1; // LFO parameter change flag
//    lcd_dbg_hex4(0, d->mult);
  } // k= ... 
} // void decode_voice()

/* copy voice data to unit from ch prop */
void copy_voice(unit_prop_t *u_p)
{
  int8_t i;
  int8_t ch = u_p->midi_ch;  
  ch_prop_t *c = &(ch_prop[ch]);
  int8_t uno = c->in_use[0];
  for (i = 0; i < 2; i++) {
    op_prop_t *o_p = &(u_p->op_prop[i]);
    o_p->vp = c->ov[i]; // copy operator voice parameter structure
//    o_p->phacc = 0;     // reset phase accumulator
//    o_p->opout = 0;
//    o_p->egmode = OFF;
  }
  u_p->uv = c->uv; // copy unit voice parameter structure
  u_p->voice_param_change = 0; // reset voice parameter change flag
  c->LFO_rate = unit_prop[uno].uv.lfo; // copy LFO rate code to channel property
  u_p->LFO_param_change = 0;   // reset LFO parameter change flag
  calc_vibrato(c); // set channel properties for vibrato
} // void copy_voice()

/*
void copy_voice(ch_prop_t *c)
{
  int uno = c->in_use[0];  
//  unit_prop[uno] = c->patch;
  c->LFO_rate = unit_prop[uno].lfo; // copy LFO rate code to channel property
  calc_vibrato(c); // set channel properties for vibrato
} // void copy_voice()
*/

/* setup voice data */
void setup_voice(ch_prop_t *c_p, int pc)
{  
  int8_t uno = c_p->in_use[0];  
  unit_prop_t *u_p = &(unit_prop[uno]);
  preload_voice(c_p, pc);
  decode_voice(c_p);
//  all_note_off(c_p);
//  copy_voice(c_p);
  calc_vol_exp(c_p);
} // void setup_voice()

/* edit voice data */
void edit_voice(ch_prop_t *c_p, int cc, int val)
{  
  int pc  = c_p->MD_progno; // get program number
  int uno = c_p->in_use[0];
  struct ma2reg *ma2 = &(c_p->vdat.op[1]); // current voice data buffer
  struct ma2reg *tab = &(((struct voicedata *)(vma2+pc))->op[1]); // original voice data pntr
  val = ((val - 0x40) >> 2); // convert (0..127) to (-16..+15)
  switch (cc) {
    case 0x49 : { // Envelope attack
      val += tab->AR;
      if      ( 1 > val) val = 1; // disable rate=0 for attack
      else if (15 < val) val = 15;
      ma2->AR = val;
      break;
    }  // case 0x49 : 
    case 0x4b : { // Envelope decay
      val += tab->DR;
      if      ( 0 > val) val = 0;
      else if (15 < val) val = 15;
      ma2->DR = val;
      break;
    }  // case 0x4b : 
    case 0x48 : { // Envelope release
      val += tab->RR;
      if      ( 0 > val) val = 0;
      else if (15 < val) val = 15;
      ma2->RR = val;
      break;
    }  // case 0x48 : 
    default: {
      break;
    } // default
  }  // switch
  decode_voice(c_p);
  copy_voice(&(unit_prop[uno]));
} // void setup_voice()
