/****************************************/
/* panelcmd.c : Process panel command   */
/*                                      */
/* NON COMMERCIAL USE ONLY              */
/*                                      */
/* 2012/04/15 for STM32F4-Discovery     */
/* 2007/03/26 Rev. 1.00 by pcm1723      */
/*                                      */
/****************************************/

#include "../common/tonegen.h"
#include <stdlib.h>

int8_t midi_map_find_next(int8_t curr_part, int8_t incdec)
{
  int8_t curr_ch, next_ch;
  curr_ch = part_to_ch[curr_part]; // map current part to midi ch
  next_ch = curr_ch;
  do {
    next_ch = (0x0f & (next_ch + incdec)); // advance (inc/dec) to next ch
  } while (MAXMIDICH > ch_to_part[next_ch]); // until free ch found
  part_to_ch[curr_part] = next_ch; // assigned new midi ch
  ch_to_part[curr_ch] = ch_to_part[next_ch];
  ch_to_part[next_ch] = curr_part;
  return (next_ch); 
} // int8_t midi_map_find_next()


void panel_cmd(int8_t sw)
{
  int8_t i;
  int8_t midi_ch_lim;
  
  midi_ch_lim = ((16 <= MAXMIDICH) ? 16 : MAXMIDICH); // MIDI CH limit
  
  if (PRESS == abs(sw)) { // key "press"ed
#if defined(LCD)
  if (sw > 0) { // increment
    panel_mode++; // advance panel
    if ((16 <= MAXMIDICH) & (CH_ASSIGN == panel_mode)) { // skip
      panel_mode++; // channel assign mode if (poly > 16);;
    } // if ((16 < ... 
    if (SYS_RESET <= panel_mode) panel_mode = MIDI_MON; //
  }
  else { // decrement
    panel_mode--; // backword panel
    if ((16 <= MAXMIDICH) & (CH_ASSIGN == panel_mode)) { // skip
      panel_mode--; // channel assign mode if (poly > 16);;
    } // if ((16 < ...
    if (MIDI_MON > panel_mode) panel_mode = SYS_RESET - 1; //
  }
  lcd_draw_panel(panel_mode);
#endif
  } // if (PRESS == abs(sw))
  else { // key "click"ed
    switch (panel_mode) { //
      case MIDI_MON  : // MIDI monitor
      case SHOW_NRPN : // show RPN/NRPN
        midi_mon_part += sw; // +1 for INC, -1 for DEC
        if (midi_ch_lim <= midi_mon_part) midi_mon_part = 0;
        if (0           >  midi_mon_part) midi_mon_part = midi_ch_lim - 1;
        break;
      case CH_ASSIGN : // MIDI channel assign
        for (i = 0; i < MAXMIDICH; i++) {
          all_note_off(&(ch_prop[i]));
        } // for (i = 0 ...
#ifdef LCD
        lcd_cur_pos = 0x10 + midi_map_find_next(midi_mon_part, sw);
#endif
/*
        for (i = 0; (i < 16) & (ch_to_part[i] != 0); i++) {} // find mapped ch 0
        i++;
        i &= 0x0f; // limit to 0..15
        for (j = 0; j < 16; j++) {
          k = (j + i) & 0x0f;
          ch_to_part[k] = j; // channel to part mapping
          if (j < MAXMIDICH) {           // valid part
            all_note_off(&(ch_prop[j])); // all note off for this part
            part_to_ch[j] = k;           // part to channel mapping
          } // if
        } // for
*/
        break;
      case SET_CONFIG : //
        if (0 < sw) { // increment
          if (MAX_CONFIG <= (++sys_mode_index)) sys_mode_index = 0;
        } else { // decrement
          if (0         >   (--sys_mode_index)) sys_mode_index = MAX_CONFIG-1;
        }
        GM_system_on();
        set_config(sys_mode_index);
        hard_reset();
        break;
      default: break;
    } // switch (panel_mode)
#ifdef LCD
    lcd_draw_panel(panel_mode);
#endif
  } // else
} // void panel_cmd()

