/****************************************************/
/*                                                  */
/* sysconf.h : System configuration define          */
/*                                                  */
/* NON COMMERCIAL USE ONLY                          */
/*                                                  */
/* 2011/04/20 Rev. 1.01 by pcm1723 (reorganized)    */
/*                                                  */
/****************************************************/

#ifndef SYSCONF_H
#define SYSCONF_H

//
// System configuration structure
//
#if defined(STM32F4XX) // STM32F4-Discovery

typedef struct {
    int16_t  polyphony;  // 
    uint32_t fs_Hz;      // sampling freq. in Hz
    int16_t  tick1ms;    // tick count for 1 ms
    int16_t  tick4ms;    // tick count for 4 ms
    int16_t  oct_ofs;    // octave offset
    int16_t  note_ofs;   // f-number table offset
    char     fs_asc[5];  // %4.1f string for fs
    int16_t  plln;       // PLLI2S N value
    int16_t  pllr;       // PLLI2S R value
  } sys_config_st_t;

#else

typedef struct {
    int16_t  polyphony;  //
    int16_t  t1div;      // Timer 1 counter load value
    int16_t  tick1ms;    // tick count for 1 ms
    int16_t  tick4ms;    // tick count for 4 ms
    int16_t  oct_ofs;    // octave offset
    int16_t  note_ofs;   // f-number table offset
    char     fs_asc[3];  // 2-digit ASCII code for fs
  } sys_config_st_t;

#endif
//
// variables
//
extern int MAXMIDICH;
extern int MAXUNITS;
extern int SEQTICK1ms;
extern int SEQTICK4ms;
extern int FS_ASC;

extern const sys_config_st_t sys_config[2][MAX_CONFIG];

//
// functions
//
extern void set_config(int mode);
extern void enable_interrupt(void);
extern void	plaInitialize(void);

#endif // SYSCONF_H
