/****************************************************/
/*                                                  */
/* unit.h : Tone generation unit define             */
/*                                                  */
/* NON COMMERCIAL USE ONLY                          */
/*                                                  */
/* 2011/04/20 Rev. 1.01 by pcm1723 (reorganized)    */
/*                                                  */
/****************************************************/

#ifndef UNIT_H
#define UNIT_H

#define UNIT_VAL_SHIFT (8)
#define UNIT_VAL_ROUND (1 << (15-UNIT_VAL_SHIFT))

/* unit propery definition */
typedef struct {
    uint32_t  ph_inc;    // f-number in linear format  
    int32_t   note_f;    // current note number with fraction 
    int32_t   porta_delta; // portamento acc increment (signed)
    int32_t   m_note_f;  // target note number with fraction
    int    waveout;   // waveform output of this unit   
    int    vel_lb;    // MIDI velocity   in log2        
    int    exp_lb;    // MIDI ch vol * MIDI expression in log2        
    int16_t  ks6_0dB;   // Level key scaling value (6.0dB/oct)
    uint8_t  on_cnt;    // multiple keyon counter         
    uint8_t  midi_note; // MIDI note number (0..127)      
    uint8_t  midi_vel;  // MIDI velocity    (0..127)      
    uint8_t  oct;       // effective octave   (0..10)                
    uint8_t  keycode;   // effective keycode  (0..11)
    int8_t  gate;      // key-ON gate flag
    int8_t  damper;    // Hold1/Damper flag
    int8_t  porta_flag;// portamento flag
    uint8_t  hold;      // CC#64 hold1/dumper flag (0..1)
    uint8_t  ksn;       // key scaling number      
    uint8_t  fb;        // modulator feedback depth       
    uint8_t  lfo;       // LFO setting                    
    uint8_t  alg;       // algorithm  
    uint8_t  midi_ch;   // MIDI ch assigned to this unit  
    unit_vparam_t uv;   // unit voice parameter (fb/LFO/alg)
    op_prop_t     op_prop[2]; // property for operator 0, 1   
    int      voice_param_change; // voice parameter change flag
    int      LFO_param_change;   // LFO parameter change flag
#if syn_CV
    int    linCV_oct; // linear CV octave part
    int    linCV_mant;// linear CV mantissa part
#endif
  } unit_prop_t ALIGN4;

extern unit_prop_t unit_prop[ABS_MAX_UNITS];

extern int16_t unit(unit_prop_t *u_p);


#endif // UNIT_H
