/****************************************************/
/*                                                  */
/* vma2.h : MA2 voice data define                   */
/*                                                  */
/* NON COMMERCIAL USE ONLY                          */
/*                                                  */
/* 2011/04/20 Rev. 1.01 by pcm1723 (reorganized)    */
/*                                                  */
/****************************************************/

#ifndef VMA2_H
#define VMA2_H

#if defined(_SH2AFPU)
  #pragma pack 4
  #pragma bit_order right
#elif defined(__RX)
  #pragma unpack
  #pragma bit_order right
#elif (defined(STM32F4XX) | defined(STM32F103))
  #pragma pack(2)
#endif

struct ma2reg {
  uint8_t KSR : 1; // Key Scaling for Rate
  uint8_t SUS : 1; // SUStain flag
  uint8_t EGT : 1; // EG Type flag
  uint8_t VIB : 1; // VIBrato flag
  uint8_t MULT: 4; // freq MULTiplier
  
  uint8_t DR  : 4; // Decay   Rate
  uint8_t RR  : 4; // Release Rate
  
  uint8_t SL  : 4; // Sustain Level
  uint8_t AR  : 4; // Attack  Rate
  
  uint8_t KSL : 2; // Key Scaling for Level
  uint8_t TL  : 6; // Total   Level
  
  uint8_t WS  : 3; // Waveform Select
  uint8_t AM  : 1; // Amplitude Modulation flag
  uint8_t DAM : 2; // Depth of Amplitude Modulation
  uint8_t DVB : 2; // Depth of ViBrato
#if (defined(_SH2AFPU) | defined(__RX) | defined(STM32F4XX))
  uint8_t pad0; // padding for 2-byte alignment
#endif
}; // struct ma2reg

struct voicedata {
  uint8_t pad0;    // padding for alignment
  uint8_t bank;    // bank LSB
  uint8_t pcno;    // program change number
  
  uint8_t ALG : 3; // ALGorithm
  uint8_t FB  : 3; // FeedBack level
  uint8_t LFO : 2; // Low Frequency Osccillator
  struct ma2reg op[2];   // MA2 operator data
}; // struct voicedata


typedef      uint8_t ma2vdat[16];
extern const ma2vdat vma2[128];

// operator decoded voice parameter structure 

typedef struct op_vparam_st {
  uint8_t  ksr;     // key scaling rate  (voice data)
  uint8_t  egt;     // EG type           (voice data)
  uint8_t  vib;     // vibrato flag      (voice data)
  uint8_t  mult;    // freq multiplier (linear 1..30) 
  uint8_t  dr;      // EG delay  rate    (voice data)
  uint8_t  rr;      // EG release rate   (voice data)
  uint16_t sl;      // EG sustain level  (voice data)   
  uint8_t  ar;      // EG attack rate    (voice data)          
  uint8_t  ksl;     // key scaling level (voice data)
  uint16_t tl;      // EG total   level  (voice data)           
  uint8_t  wavesel; // wave select       (voice data)          
  uint8_t  am;      // AM flag           (voice data)
  uint8_t  dam;     // AM depth          (voice data)
  uint8_t  dvb;     // vibrato depth     (voice data)
  uint8_t  carrf;   // carrier flag 
  uint8_t  op_num;  // Operator number
  int16_t (*op_func_p)(void *o_p);
} op_vparam_t, *op_vparam_p;

/* unit voice parameter */

typedef struct unit_vparam_st {
    uint8_t fb;        // modulator feedback depth       
    uint8_t lfo;       // LFO setting                    
    uint8_t alg;       // algorithm  
} unit_vparam_t, *unit_vparam_p;

#if (defined(STM32F4XX) | defined(STM32F103))
  #pragma pack()
#endif

#endif // VMA2_H
